@@ -46,18 +46,30 @@ typedef enum {
 	CI_MVZ_NN  				= 18,
 	CI_MVZ_GAUSSIAN			= 19,
 	CI_UDF_ADP_MVZ_NN		= 20,
-	CI_MVZ_NN_GRID_BW		= 21
+	CI_MVZ_NN_GRID_BW		= 21,
+	K_SAMPLE_DDP_M2			= 22,
+	K_SAMPLE_DDP_M3			= 23,
+	K_SAMPLE_DDP			= 24,
+	GOF_DDP_M2		        = 25,
+	GOF_DDP_M3		        = 26,
+	GOF_DDP			        = 27,
+	K_SAMPLE_EXISTING		= 100,
+	GOF_EXISTING		    = 101
 } TestType;
 
-#define IS_UDF_TEST(tt) ((tt) == UDF_SPR_OBS    || (tt) == UDF_SPR_ALL    || (tt) == UDF_PPR_22_OBS || \
-		                 (tt) == UDF_PPR_22_ALL || (tt) == UDF_PPR_33_OBS || (tt) == UDF_PPR_33_ALL || \
-		                 (tt) == UDF_TPR_OBS    || (tt) == UDF_TPR_ALL    || (tt) == UDF_SPPR_OBS   || \
-		                 (tt) == UDF_SPPR_ALL   || (tt) == UDF_DDP_OBS    || (tt) == UDF_DDP_ALL    || \
+#define IS_UDF_TEST(tt) ((tt) == UDF_SPR_OBS       || (tt) == UDF_SPR_ALL     || (tt) == UDF_PPR_22_OBS  || \
+		                 (tt) == UDF_PPR_22_ALL    || (tt) == UDF_PPR_33_OBS  || (tt) == UDF_PPR_33_ALL  || \
+		                 (tt) == UDF_TPR_OBS       || (tt) == UDF_TPR_ALL     || (tt) == UDF_SPPR_OBS    || \
+		                 (tt) == UDF_SPPR_ALL      || (tt) == UDF_DDP_OBS     || (tt) == UDF_DDP_ALL     || \
 		                 (tt) == CI_UDF_ADP_MVZ_NN)
 
 #define IS_CI_MVZ_TEST(tt) ((tt) == CI_MVZ_NN         || (tt) == CI_MVZ_GAUSSIAN || \
 		                    (tt) == CI_UDF_ADP_MVZ_NN || (tt) == CI_MVZ_NN_GRID_BW)
 
+#define IS_K_SAMPLE_DDP(tt) ((tt) == K_SAMPLE_DDP_M2 || (tt) == K_SAMPLE_DDP_M3 || (tt) == K_SAMPLE_DDP)
+
+#define IS_GOF_DDP(tt) ((tt) == GOF_DDP_M2 || (tt) == GOF_DDP_M3 || (tt) == GOF_DDP)
+
 //#define UDF_ALLOW_DEGENERATE_PARTITIONS
 #define UDF_NORMALIZE
 
@@ -69,7 +81,8 @@ typedef std::vector< std::vector<dbl_int_pair> > dbl_int_pair_matrix;
 struct ExtraParams {
 	double w_sum;
 	double w_max;
-	int K; // number of unique y values, also used for HHGCI NN kernel width, and for order of ADP/DDP partitions
+	int K; // number of unique y values in K-sample test, also used for HHGCI NN kernel width, and for order of ADP/DDP partitions
+	int M; // order of ADP/DDP partition for the K-sample ADP/DDP test
 	int* y_counts; // counts observed for each unique y value, sorted by y value
 	bool correct_mi_bias;
 	double sig;
@@ -82,6 +95,7 @@ struct ExtraParams {
 		w_sum = 0;
 		w_max = 0;
 		K = 0;
+		M = 0;
 		y_counts = NULL;
 		correct_mi_bias = false;
 		sig = 0;
