@@ -53,6 +53,14 @@ protected:
 	void hhg_ci_mvz_gaussian(void);
 	void hhg_ci_udf_adp_mvz_nn(void);
 	void hhg_ci_mvz_nn_grid(void);
+	void hhg_k_sample_existing(void);
+	void hhg_k_sample_ddp_m2(void);
+	void hhg_k_sample_ddp_m3(void);
+	void hhg_k_sample_ddp(void);
+	void hhg_gof_existing(void);
+	void hhg_gof_ddp_m2(void);
+	void hhg_gof_ddp_m3(void);
+	void hhg_gof_ddp(void);
 
 	void other_stats_two_sample(void);
 	void other_stats_k_sample(void);
@@ -61,13 +69,16 @@ protected:
 	void other_stats_ci(void);
 
 	void accumulate_2x2_contingency_table(double a00, double a01, double a10, double a11, double nrmlz, double reps);
+	void accumulate_local_stats(double chi, double like, double emin);
 	void hhg_gen_inversions(int *permutation, int *source, int *inversion_count, int dim);
 	void hhg_gen_merge(int *permutation, int *source, int *inversion_count, int dim);
+	void compute_single_integral(int n, double* xx, double* yy);
 	void compute_double_integral(int n, double* xx, double* yy);
 	int count_sample_points_in_rect(int xl, int xh, int yl, int yh);
 	double count_ddp_with_given_cell(int xl, int xh, int yl, int yh);
 	double count_adp_with_given_cell(int xl, int xh, int yl, int yh);
 	void precompute_adp(void);
+	void precompute_adp_k_sample(void);
 	double my_choose(int n, int k);
 	int my_rand(int lo, int hi);
 
@@ -89,6 +100,7 @@ protected:
 	double* z;
 	dbl_int_pair_matrix *sorted_dx, *sorted_dy, *sorted_dz;
 	int K; // k-sample test: number of unique y values; DDP/ADP: partition order
+	int M; // in the k-sample ADP/DDP test, this is the partition order and K is the number of unique y values
 	int* y_counts; // counts observed for each unique y value, sorted by y value
 	double w_sum;
 	double w_max;
@@ -114,6 +126,7 @@ protected:
 	int *hhg_gen_left_buffer, *hhg_gen_right_buffer, *hhg_gen_left_source_buffer, *hhg_gen_right_source_buffer;
 	bool should_randomize;
 
+	double kahan_c_chi, kahan_c_like;
 	int* double_integral;
 	int dintegral_zero_based_idxs;
 	int dintegral_pn;
@@ -121,6 +134,7 @@ protected:
 	int ng_chi, ng_like;
 	bool correct_bias;
 	dbl_int_pair_vector nn_sorted_x, nn_sorted_y;
+	double* null_dist;
 
 	struct dbl_dbl_int {
 		double x;
